unit BotDemo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, BotUnit, LogUnit, BotCPU;

type
  TMainForm = class(TForm)
    MainTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MainTimerTimer(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
  end;

var
  MainForm: TMainForm;
  DblBuffer: TBitmap;
  Maze: array [-1..12,-1..16] of boolean;
  Keys: array [0..255] of boolean;

  Objects: array of TGameObject;

procedure log(st: string);

implementation

{$R *.dfm}

//   

procedure UpdateGame();
var
  i: integer;
begin
  for i := 0 to high(Objects) do
    if Objects[i] <> nil then
      Objects[i].Update();
end;

//   

procedure PrepareBuffer();
var
  x,y,i: integer;
begin
  with DblBuffer do
  begin
    for y := 0 to 11 do
      for x:= 0 to 15 do
      begin
        if Maze[y,x] then Canvas.Brush.Color:=clRed
          else Canvas.Brush.Color:=clBlue;
        Canvas.FillRect(Rect(x*32,y*32,(x+1)*32,(y+1)*32));
      end;
    Canvas.Pen.Color:=clWhite;
    Canvas.Pen.Width:=1;
    for x := 1 to Width div 32 do
    begin
      Canvas.MoveTo(x*32,0);
      Canvas.LineTo(x*32,Height);
    end;
    for y := 1 to Height div 32 do
    begin
      Canvas.MoveTo(0,y*32);
      Canvas.LineTo(Width,y*32);
    end;
  end;
  for i:=0 to high(Objects) do
    if Objects[i] <> nil then
      Objects[i].Draw();
end;

//  

procedure TMainForm.FormCreate(Sender: TObject);
var
  i: integer;
begin
  Fillchar(keys,sizeof(keys),0);
  Randomize();
  ClientHeight:=128*3;
  ClientWidth:=128*4;
  Left:=64;
  Top:=64;
  DblBuffer:=TBitmap.Create;
  with DblBuffer do
  begin
    width:=ClientWidth;
    height:=ClientHeight;
  end;
  // 
  fillchar(Maze,sizeof(Maze),0);
  for i := -1 to 16 do  //  (  )
  begin
    Maze[-1,i]:=true;
    Maze[12,i]:=true;
  end;
  for i := -1 to 12 do  //  (  )
  begin
    Maze[i,-1]:=true;
    Maze[i,16]:=true;
  end;
  for i := 1 to 16*12 div 8 do
    Maze[random(12),random(16)]:=true;
  // 
  THumanPlayer.Create;
  TCPUPlayer.Create;
end;

//  

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  i: integer;
begin
  DblBuffer.Free;
  for i := 0 to high(Objects) do
    Objects[i].Free;
end;

// ,     

procedure TMainForm.MainTimerTimer(Sender: TObject);
begin
  UpdateGame();
  PrepareBuffer();
  Canvas.Draw(0,0,DblBuffer);
end;

// 

procedure TMainForm.FormPaint(Sender: TObject);
begin
  Canvas.Draw(0,0,DblBuffer);
end;

// 

procedure log(st: string);
var
  f: textfile;
begin
  if LogForm.LogMemo.Lines.Count > 128 then
    LogForm.LogMemo.Lines.Delete(0);
  LogForm.LogMemo.Lines.Add(TimeToStr(Now)+' - '+st);
  assignfile(f,'log.txt'); append(f);
  writeln(f,st);
  closefile(f);
end;

//   

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Keys[key]:=true;
  if key = VK_ESCAPE then close;
end;

//   

procedure TMainForm.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Keys[Key]:=false;
end;

end.

