unit BotUnit;
interface
uses Graphics, Classes, Windows, SysUtils;
const
  pWidth = 6;                                         // 
  pLength = 12;                                       // 
  pFireTime = 8;                                      // 
  pLife = 4;                                          //  
  cor = -(2 * 3.1415926535897932384626433832795)/64;  //
  pSpeed = 2.1;                                       // 
  rSpeed = 3.7;                                       // 
  rWidth = 3;                                         // 

var
  objectCounter: integer = 0;

type

//,   

  TGameObject = class(TObject)
  protected
    xPos, yPos: double;
    dir: integer;
    fRadius: integer;
  public
    property Radius: integer read fRadius;            //
    property xPosition: double read xPos;             // x
    property yPosition: double read yPos;             // y
    property direction: integer read dir;             // 
                                                      //

    procedure Draw(); virtual; abstract;              // 
    procedure Update(); virtual; abstract;            // 
                                                      // 
                                                      //
    function Name(): string; virtual; abstract;       //  

    constructor Create;
    destructor Destroy(); override;
  end;

//,  

  TPlayer = class(TGameObject)
  protected
    fColor: TColor;
    toFire: integer;
    life: integer;
  public
    search: Integer;
    procedure onKilled; virtual;                      //  
    procedure onHit; virtual;                         //  
    procedure onOtherPlayerKilled(); virtual;         //  
                                                      // 
    procedure onOtherPlayerFired(); virtual;          //  
                                                      // 
    procedure CheckColission();                       // 
    procedure Draw(); override;                       // 
    procedure GeneratePosition();                     //  
                                                      // 
    procedure Restart();                              // 
                                                      //  
                                                      // 
                                                      //
    procedure DoStep();                               // 
    procedure DoLeft();                               // 
    procedure DoRight();                              // 
    procedure DoFire();                               //
  end;

//,  

  TMissle = class(TGameObject)
  private
    ndx: integer;
  public
    function Name(): string; override;                //  
    procedure Draw(); override;                       // 
    procedure Update(); override;                     // 
                                                      // 
    constructor Create(sender: TPlayer);
    destructor Destroy(); override;
  end;

//,  ,  

  THumanPlayer = class(TPlayer)
  public
    constructor Create;
    function Name(): string; override;                //  

    procedure Update(); override;                     // 
                                                      // 
  end;

implementation
uses BotDemo;

destructor TGameObject.Destroy();
var
  i: integer;
begin
  i:=0;
  while(i<length(Objects))do
  begin
    if(Objects[i]=self) then
      Objects[i]:=nil;
    inc(i);
  end;
  inherited;
end;

constructor TGameObject.Create;
var
  i, ndx: integer;
begin
  inherited;
  ndx:=-1;
  for i := 0 to high(Objects) do
    if Objects[i] = nil then ndx:=i;
  if ndx < 0 then
  begin
    ndx:=length(Objects);
    setLength(Objects,length(Objects)+1);
  end;
  Objects[ndx]:=self;
end;

//  

procedure TPlayer.onKilled;
begin
end;

//  

procedure TPlayer.onHit;
begin
end;

//    

procedure TPlayer.onOtherPlayerKilled();
begin
end;

//    

procedure TPlayer.onOtherPlayerFired();
begin
end;

// 

procedure TPlayer.Draw();
var
  i: integer;
begin
  with DblBuffer.Canvas do
  begin
    Pen.Width:=1;
    Pen.Color:=clWhite;
    Pen.Width:=2;
    for i := 1 to life do
    begin
      MoveTo(Round(xPos),Round(yPos));
      LineTo(Round(xPos+pLength*cos((dir-3-i*5)*cor)),Round(yPos+pLength*sin((dir-3-i*5)*cor)));
      MoveTo(Round(xPos),Round(yPos));
      LineTo(Round(xPos+pLength*cos((dir+3+i*5)*cor)),Round(yPos+pLength*sin((dir+3+i*5)*cor)));
    end;
    Pen.Color:=fColor;
    Brush.Color:=fColor;
    Ellipse(Rect(round(xPos)-pWidth,round(yPos)-pWidth,round(xPos)+pWidth,round(yPos)+pWidth));
    MoveTo(Round(xPos),Round(yPos));
    Pen.Width:=3;
    LineTo(Round(xPos+pLength*cos(dir*cor)),Round(yPos+pLength*sin(dir*cor)));
  end;
end;

// 

procedure TPlayer.CheckColission();
var
  i: integer;
begin
  for i := 0 to high(Objects) do
    if (Objects[i] <> nil) and (Objects[i] is TPlayer) and (Objects[i] <> self) then
    begin
      if sqrt(sqr(xPos-Objects[i].xPos)+sqr(yPos-Objects[i].yPos)) <= self.fRadius*2  then
      begin
        self.Restart();
        (Objects[i] as TPlayer).Restart;
      end;
    end;
end;

//    

procedure TPlayer.GeneratePosition();
var
  x,y: integer;
begin
  life:=pLife;
  repeat
    x:=random(4*128);
    y:=random(3*128);
    dir:=random(16);
  until not Maze[y shr 5, x shr 5];
  xPos:=x;
  yPos:=y;
end;

// 

procedure TPlayer.DoLeft();
begin
  dir:=(dir+1) mod 64;
end;

// 

procedure TPlayer.DoRight();
begin
  dir:=(dir+64-1) mod 64;
end;

// 

procedure TPlayer.DoStep();
var
  x,y: double;
begin
  x:=xPos+cos(dir*cor)*pSpeed;
  y:=yPos+sin(dir*cor)*pSpeed;
  if(x< 0) then x:=0.01;
  if(y< 0) then y:=0.01;
  if Maze[round(y) shr 5,round(x) shr 5] then
  begin
    log(' "'+name()+'"   ');
  end else
  begin
    xPos:=x;
    yPos:=y;
  end;
end;

//

procedure TPlayer.DoFire();
var
  i: integer;
begin
  if toFire > pFireTime then
  begin
    TMissle.Create(self);
    toFire:=0;
    for i := 0 to high(Objects) do
      if (Objects[i] <> nil) and (Objects[i] is TPlayer) and (Objects[i] <> self) then (Objects[i] as TPlayer).onOtherPlayerFired();
  end;
end;

//      

procedure TPlayer.Restart();
var
  i: integer;
begin
  dec(Life);
  if Life <= 0 then
  begin
    onKilled();
    log(' "'+name()+'"       ');
    for i := 0 to high(Objects) do
      if (Objects[i] <> nil) and (Objects[i] is TPlayer) and (Objects[i] <> self) then (Objects[i] as TPlayer).onOtherPlayerKilled();
    GeneratePosition();
  end else
  begin
    onHit();
    log(' "'+name()+'"  ');
  end;
end;

constructor THumanPlayer.Create;
begin
  inherited;
  fRadius:=pWidth;
  fColor:=clYellow;
  GeneratePosition();
end;

//  

function THumanPlayer.Name(): string;
begin
  result:='-';
end;

//  

procedure THumanPlayer.Update();
begin
  inc(toFire);
  if Keys[VK_LEFT] then DoLeft();
  if Keys[VK_RIGHT] then DoRight();
  if Keys[VK_UP] then DoStep();
  if Keys[VK_SPACE] then DoFire();
  CheckColission();
end;

constructor TMissle.Create(sender: TPlayer);
begin
  inherited Create;
  fRadius:=rWidth;
  ndx:=objectCounter;
  inc(objectCounter);
  dir:=sender.dir;
  xPos:=sender.xPos+cos(dir*cor)*(rSpeed+pSpeed)*2;
  yPos:=sender.yPos+sin(dir*cor)*(rSpeed+pSpeed)*2;
end;

destructor TMissle.Destroy();
begin
  inherited;
end;

//  

function TMissle.Name(): string;
begin
  result:=''+inttostr(ndx);
end;

// 

procedure TMissle.Draw();
begin
  with DblBuffer.Canvas do
  begin
    Pen.Color:=clBlack;
    Brush.Color:=clBlack;
    Ellipse(Rect(round(xPos)-rWidth,round(yPos)-rWidth,round(xPos)+rWidth,round(yPos)+rWidth));
  end;
end;

//  

procedure TMissle.Update();
var
  i: integer;
begin
  xPos:=xPos+cos(dir*cor)*rSpeed;
  yPos:=yPos+sin(dir*cor)*rSpeed;
  if (xPos < 0 ) or (yPos < 0) then
  begin
    self.Free;
    exit;
  end;
  if Maze[round(yPos) shr 5,round(xPos) shr 5] then
  begin
  //   
    log(' "'+name()+'" :   ');
    self.Free;
  end else
  begin
  //   
    for i := 0 to high(objects) do
      if (objects[i] <> nil) and (objects[i] <> self) then
        if sqrt(sqr(xPos-objects[i].xPos)+sqr(yPos-Objects[i].yPos)) <= radius+Objects[i].Radius then
        begin
          log(' "'+name()+'" :    ("'+Objects[i].Name()+'")');
          if Objects[i] is TMissle then Objects[i].Free else
          begin
            (Objects[i] as TPlayer).Restart;
          end;
          self.Free;
          exit;
        end;
  end;
end;

end.

